/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package nsk.share.jvmti.RetransformClasses.LinearHierarchy;

public class Class3 extends Class4 {
    byte var_1;
    double var_2 = 9.323170900039764E307;
    final double var_3 = 8.336052451198062E305;
    final short var_4 = 23053;
    String var_5 = "tookgfu";
    final double var_6 = 9.238786778915182E307;
    double var_7;


    public Class3()
    {
        try {
            this.test();
        } catch (Throwable e) {}
    }

    private void test()
    {
        {
            {
                var_2 *= true ? 'n' : 'O';
                var_2 -= var_4;
                var_1 = (var_1 = (var_1 = (var_1 = (byte)'q')));
                var_1 = (var_1 = true ? (byte)191666986 : (byte)(long)4.691435472427452E307);
                var_5 = var_5;
                var_5 = var_5;
            }
            var_2 /= (long)'_';
            var_1 = (var_1 = (byte)7676653265587243008L);
            var_1 = (var_1 = (var_1 = (var_1 = (var_1 = (byte)863982021))));
            double var_8;
            long var_9 = 0L;
            final String var_10 = "hw";
            do
            {
                var_5 = var_5;
                var_9++;
                var_1 = (var_1 = (var_1 = (byte)(2.6990877E38F - - ((byte)var_2))));
            } while ((false & !false | (true ? false : true & !false)) ^ true && var_9 < 33);
            (! (var_4 >= 1.2995355302120393E308) ? var_5 : var_10).lastIndexOf("pnpykum");
            var_1 = (byte)'p';
            var_5.substring(205902665 + 1439254027, false ? var_4 : + ((byte)1.1194373736770853E308));
            var_5 = "";
        }
        final boolean var_11 = false;
        if (var_4 >= 's' ? var_11 : var_11)
        {
            var_5 = (var_5 = "ruutjov");
            var_1 = (var_1 = (var_1 = var_11 && false ? (byte)8005016235883617280L : (byte)7159126825765328896L));
            double var_12 = 0;
            (var_5 = (var_5 = "yyja")).compareToIgnoreCase(var_5 = "xcngthf");
            do
            {
                "owlad".substring((int)(var_1 = (byte)8.486030365558038E305) % 2926964179631979520L > var_4 ? (int)'[' / (var_4 * 1856630598) : 's');
                var_12++;
                var_5 = var_5;
            } while (var_12 < 29);
            var_7 = 'b' & (var_1 = (var_1 = (byte)692202052));
            var_5.codePointBefore((! ((true ^ true) & var_11) ? (var_1 = (byte)var_12) : (byte)1.2120844278068918E308) + (var_1 = true ^ (!false | var_11) ? (byte)1630258429027521536L : (var_1 = (byte)var_6)));
            var_1 = (var_1 = (var_1 = (byte)966477819));
            var_2 *= var_4;
        }
        else
        {
            boolean var_13;
        }
        long var_14 = 0L;
        {
            var_1 = (var_1 = (var_1 = (var_1 = (var_1 = (var_1 = (var_1 = (byte)1267533979))))));
        }
        var_1 = (var_1 = (var_1 = (byte)var_4));
        var_2 *= (var_1 = (byte)-5.8943603E37F) / (char)(var_4 + var_4);
        {
            long var_15 = var_14 << 'K';
        }
        var_1 = var_11 ? (byte)2.3822595E38F : (byte)-var_14;
        var_5 = "piedleg";
        if (false)
        {
            var_5 = var_5;
        }
        else
        {
            var_2 -= 4.1427217E37F;
            var_5 = (var_5 = (var_5 = (var_5 = "bmth")));
        }
        var_2 *= var_11 | (var_11 | var_4 >= 6760721935678482432L) ? var_4 : 1660713514769498112L;
        var_2 += var_14;
        var_2 /= var_4 % (byte)2.2813638E38F;
        var_2 += (double)var_4;
        var_2 %= 5231114328220607488L;
        do
        {
            var_1 = (true ? false ^ true : var_11) ? (byte)(+ ((byte)'c') * -1.8563719E38F) : (var_1 = (byte)4.9655233E37F);
            var_5 = "mgaryhi";
            var_5.substring('n' - 'h', var_11 ? ~ ((byte)-2054423460406148096L) : var_4);
            var_2 -= 'u' * 5718883558015662080L;
            var_1 = (var_1 = (var_1 = (byte)2.4555978E38F));
            var_5 = (var_5 = var_5);
            var_2 = 4.324052E37F <= (var_2 %= 1978604915) ? 's' : 'p';
            final byte var_16 = 123;
            (var_5 = (var_5 = "qlw")).substring(var_4 / 1759986631);
            var_14++;
            var_5 = !false | (false && !false) ? (var_5 = (var_5 = var_5)) : (var_5 = (var_5 = "gwgweaw"));
            var_7 = var_11 ? var_16 : var_4;
            var_2 %= var_2;
            {
                var_1 = false ? (var_1 = (var_1 = var_16)) : var_16;
            }
            var_1 = var_16;
            var_2 %= (var_1 = (var_1 = (var_1 = var_16)));
            final int var_17 = 'P' % var_4;
            var_1 = (var_1 = var_16);
            switch ((char)+ + ~ - + ((long)var_2))
            {
                case 'J':

            }
            var_1 = (var_1 = (var_1 = (var_1 = var_16)));
            var_1 = (var_1 = var_16);
        } while (var_14 < 346);
        long var_18 = 0L;
        var_5 = "ci";
        do
        {
            Object var_19;
            var_18++;
            var_2 += (var_1 = (var_1 = (var_1 = var_11 ? (byte)var_14 : (byte)var_4)));
            var_1 = (var_1 = (byte)2.383327444966934E307);
            var_2 *= var_4 / 1.6500017E37F;
        } while (var_18 < 63 && var_11);
        long var_20 = 0L;
        var_1 = var_11 ? (var_1 = (var_1 = (byte)var_14)) : (byte)~var_18;
        while (var_3 >= ((byte)1.586505E38F | (byte)var_2) && var_20 < 268)
        {
            char var_21;
            var_20++;
            var_1 = (byte)9.660374E36F;
            double var_22 = var_6;
        }
        var_1 = (true ? ! !false | true : false) ? (byte)var_2 : (var_1 = (var_1 = (var_1 = (byte)- ((byte)var_4))));
        float var_23 = 0F;
        var_2 %= -var_20;
        do
        {
            var_2 %= 1.1852769E38F;
            var_23++;
            var_5 = "rtjepio";
        } while (var_23 < 72 && var_11);
        var_5 = "dtjup";
        var_5 = var_5;
        float var_24 = 0F;
        var_2 = var_2;
        while (var_24 < 267)
        {
            var_5 = "u";
            var_24++;
            var_2 -= (var_1 = (var_1 = (var_1 = (var_1 = (var_1 = (byte)2.9356384E38F)))));
            var_2 *= (var_1 = (var_1 = (byte)'X'));
        }
        var_5 = "ijtugph";
        var_1 = (var_5 = "tucesya").endsWith("g") ? (!false & !var_11 ? (byte)3.251367E38F : (byte)3.1438577E38F) : (var_1 = (byte)var_4);
    }
}
