/*
 * Copyright (c) 2017, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */


import java.lang.BootNativeLibrary;

/*
 * This is called from FindClassFromBoot class.
 */
public class BootLoaderTest {
    public static void main(String... args) throws Exception {
        testJNIFindClass("java/lang/String", String.class);
        testJNIFindClass("java/lang/BootNativeLibrary", BootNativeLibrary.class);
        testJNIFindClass("BootLoaderTest", null);
    }

    /*
     * Call JNI FindClass with null loader as the context
     */
    static void testJNIFindClass(String name, Class<?> expected) {
        Class<?> c = BootNativeLibrary.findClass(name);
        if (c != expected) {
            throw new RuntimeException("FindClass " + c + " expected: " + expected);
        }
    }
}
