/*
 * Copyright (c) 2021, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package compiler.lib.ir_framework;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

/**
 * This annotation is used for test methods (see {@link Test}) to specify what values should be passed as arguments.
 * One can either specify the individual arguments with values (see {@link Argument}), or use
 * a setup method (see {@link Setup}) to define more complex arguments and/or even set fields values.
 * This annotation can only be applied to a <b>normal test</b>.
 *
 * @see Argument
 * @see Test
 * @see Check
 */
@Retention(RetentionPolicy.RUNTIME)
public @interface Arguments {
    /**
     * Get the argument values.
     */
    Argument[] values() default {};
    /**
     * Get the setup method name.
     */
    String setup() default "";
}
