/*
 * Copyright (c) 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * @bug 8330795
 * @summary Check that Reduce Allocation Merges doesn't crash when CompressedClassPointers
 *          is disabled and there is an access to Klass "field" through the phi.
 * @requires vm.bits == 64 & vm.flagless & vm.compiler2.enabled & vm.opt.final.EliminateAllocations
 * @run main/othervm -XX:CompileCommand=dontinline,*TestReduceAllocationAndLoadKlass*::test
 *                   -XX:CompileCommand=compileonly,*TestReduceAllocationAndLoadKlass*::test
 *                   -XX:CompileCommand=compileonly,*Shape*::*init*
 *                   -XX:CompileCommand=compileonly,*Point*::*init*
 *                   -XX:CompileCommand=exclude,*TestReduceAllocationAndLoadKlass*::dummy
 *                   -XX:-TieredCompilation
 *                   -XX:-UseCompressedClassPointers
 *                   -Xbatch
 *                   -Xcomp
 *                   compiler.c2.TestReduceAllocationAndLoadKlass
 */

package compiler.c2;

public class TestReduceAllocationAndLoadKlass {
    public static void main(String[] args) {
        Point p = new Point();
        Line q = new Line();

        test(true);
        test(false);
    }

    static Class test(boolean cond) {
        Object p = cond ? dummy() : new Line();
        return p.getClass();
    }

    static Point dummy() { return new Point(); }

    static class Shape { }
    static class Point extends Shape { }
    static class Line extends Shape { }
}
