/*
 * Copyright (c) 2022, Arm Limited. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * @test
 * @bug 8286125
 * @summary C2: "bad AD file" with PopulateIndex on x86_64
 *
 * @run main/othervm -Xbatch -XX:MaxVectorSize=16 compiler.vectorization.TestReplicateLoopIV
 */

package compiler.vectorization;

public class TestReplicateLoopIV {

    public static long[] arr = new long[400];

    public static void bar() {
        for (int i = 304; i > 15; i -= 3) {
            int c = 16;
            do {
                for (int t = 1; t < 1; t++) {}
                arr[c + 1] >>= i;
            } while (--c > 0);
        }
    }

    public static void main(String[] args) {
        int iter = 100000;
        while (iter-- > 0) {
            bar();
        }
    }
}

