/*
 * Copyright (c) 2016, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package optionsvalidation;

import java.lang.ref.WeakReference;

class JVMStartup {
    private static volatile WeakReference<Object> weakRef;

    private static synchronized void createWeakRef() {
        Object o = new Object();
        weakRef = new WeakReference<>(o);
    }

    /* Simple method to test that java start-up. Used for testing options. */
    public static void main(String[] args) throws Exception {
        byte[] garbage = new byte[8192];
        int i = 0;
        createWeakRef();
        do {
            garbage = new byte[8192];
            i++;
            /* Initiate GC after 5 iterations */
            if (i > 5) {
                System.gc();
            }
        } while(weakRef.get() != null);
        System.out.println("Java start-up!");
    }
}
