/*
 * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
/*
 * @test
 * @bug 8275337
 * @run main/othervm -Xcomp -XX:TieredStopAtLevel=1 compiler.c1.Test8275337
 */


package compiler.c1;

public class Test8275337 {
    public static final int N = 400;

    public static void mainTest() {
        int iArr1[] = new int[N];
        float fArr1[][] = new float[N][N];

        for (int i = 9; i < 171; i++) {
            int z;
            try {
                z = i % i;
            } catch (ArithmeticException a_e) {}
            for (int j = 1; j < 155; ++j) {
                fArr1[j - 1][i] -= 1;
                iArr1[i - 1] = 1;
            }
            for (int j = 4; j < 155; j++) {
                for (int k = 1; k < 2; ++k) {
                    iArr1[i - 1] += 1;
                    fArr1[k - 1][j] -= 2;
                }
            }
        }
    }
    public static void main(String[] strArr) {

        try {
            for (int i = 0; i < 10; i++) {
                mainTest();
            }
         } catch (Exception ex) {
         }
    }
}
