/*
 * Copyright (c) 2021, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

class SimpleLambda {
    public Runnable getRunnable() {
        return () -> {};
    }
    public OldProvider getProvider() {
        return () -> {
            return null;
        };
    }
}

public class RedefineCallerClass {

    public static String newClass =
        " class SimpleLambda { " +
        "     public Runnable getRunnable() { " +
        "         return () -> {}; " +
        "     } " +
        "     public OldProvider getProvider() { " +
        "         return () -> { " +
        "             return null; " +
        "         }; " +
        "     } " +
        " } ";

    public static void main(String args[]) throws Exception {
      String mode = "both";
      if (args.length == 1) {
          mode = args[0];
      }
      SimpleLambda s = new SimpleLambda();
      if (mode.equals("both") || mode.equals("useOldInf")) {
          System.out.println(s.getProvider());
      } else {
          System.out.println(s.getRunnable());
      }
      if (mode.equals("both") || mode.equals("redefineCaller")) {
          RedefineClassHelper.redefineClass(s.getClass(), newClass);
      }
    }
}
