/*
 * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * @test
 * @bug 8255466
 * @summary unsafe access to static field causes crash
 * @modules java.base/jdk.internal.misc
 *
 * @run main/othervm -Xcomp -XX:CompileCommand=compileonly,TestUnsafeStaticFieldAccess::* TestUnsafeStaticFieldAccess
 *
 */

import jdk.internal.misc.Unsafe;
import java.lang.reflect.Field;

public class TestUnsafeStaticFieldAccess {
    private static final Unsafe UNSAFE = Unsafe.getUnsafe();
    private static final long offset;
    private static volatile Class<?> clazz;

    private static int field;

    static {
        long o = 0;
        for (Field f : TestUnsafeStaticFieldAccess.class.getDeclaredFields()) {
            if (f.getName().equals("field")) {
                o = UNSAFE.staticFieldOffset(f);
                break;
            }
        }
        offset = o;
        clazz = TestUnsafeStaticFieldAccess.class;
    }


    public static void main(String[] args) {
        for (int i = 0; i < 12000; i++) {
            UNSAFE.getInt(clazz, offset);
        }
    }
}
