/*
 * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package nsk.jdi.VirtualMachineManager.connectedVirtualMachines;

import nsk.share.*;
import nsk.share.jpda.*;
import nsk.share.jdi.*;

import com.sun.jdi.*;
import java.util.*;
import java.io.*;

import com.sun.jdi.event.*;
import com.sun.jdi.request.*;

/**
 * The test for the implementation of an object of the type     <BR>
 * VirtualMachineManager.                                       <BR>
 *                                                              <BR>
 * The test checks up that results of the method                <BR>
 * <code>com.sun.jdi.VirtualMachineManager.connectedVirtualMachines()</code>                    <BR>
 * complies with its specification.                             <BR>
 * <BR>
 * Test case includes only one debuggee.                        <BR>
 * The test checks up that :                                    <BR>
 * - after launching debugged Virtual Machine the tested method <BR>
 *   returns List containing one element of                     <BR>
 *   a VirtualMachine type which is equal to a mirror of        <BR>
 *   the debuggee already got after launching;                  <BR>
 * - after the degugger terminates the debuggee with the method <BR>
 *   VirtualMachine.exit(),                                     <BR>
 *   the call to the method returns an empty List.              <BR>
 */

public class convm002 {

    //----------------------------------------------------- templete section
    static final int PASSED = 0;

    static final int FAILED = 2;

    static final int PASS_BASE = 95;

    //----------------------------------------------------- templete parameters
    static final String sHeader1 = "\n==> nsk/jdi/VirtualMachineManager/connectedVirtualMachines/convm002  ",
            sHeader2 = "--> debugger: ", sHeader3 = "##> debugger: ";

    //----------------------------------------------------- main method

    public static void main(String argv[]) {
        int result = run(argv, System.out);
        if (result != 0) {
            throw new RuntimeException("TEST FAILED with result " + result);
        }
    }

    public static int run(String argv[], PrintStream out) {
        return new convm002().runThis(argv, out);
    }

    //--------------------------------------------------   log procedures

    private static Log logHandler;

    private static void log1(String message) {
        logHandler.display(sHeader1 + message);
    }

    private static void log2(String message) {
        logHandler.display(sHeader2 + message);
    }

    private static void log3(String message) {
        logHandler.complain(sHeader3 + message);
    }

    //  ************************************************    test parameters

    private String debuggeeName = "nsk.jdi.VirtualMachineManager.connectedVirtualMachines.convm002a";

    //====================================================== test program
    //------------------------------------------------------ common section

    static ArgumentHandler argsHandler;

    static int waitTime;

    static VirtualMachine vm = null;

    static EventRequestManager eventRManager = null;

    static EventQueue eventQueue = null;

    static EventSet eventSet = null;

    static int testExitCode = PASSED;

    static final int returnCode0 = 0;

    static final int returnCode1 = 1;

    static final int returnCode2 = 2;

    static final int returnCode3 = 3;

    static final int returnCode4 = 4;

    //------------------------------------------------------ methods

    private int runThis(String argv[], PrintStream out) {

        Debugee debuggee;

        argsHandler = new ArgumentHandler(argv);
        logHandler = new Log(out, argsHandler);
        Binder binder = new Binder(argsHandler, logHandler);

        if (argsHandler.verbose()) {
            debuggee = binder.bindToDebugee(debuggeeName + " -vbs");
        } else {
            debuggee = binder.bindToDebugee(debuggeeName);
        }

        waitTime = argsHandler.getWaitTime();

        IOPipe pipe = new IOPipe(debuggee);

        debuggee.redirectStderr(out);
        log2(debuggeeName + " debuggee launched");
        debuggee.resume();

        String line = pipe.readln();
        if ((line == null) || !line.equals("ready")) {
            log3("signal received is not 'ready' but: " + line);
            return FAILED;
        } else {
            log2("'ready' recieved");
        }

        vm = debuggee.VM();

        //------------------------------------------------------  testing section
        log1("      TESTING BEGINS");

        VirtualMachineManager vmm = null;

        List<VirtualMachine> connectedVM;
        VirtualMachine vm1 = null;

        int size = 0;

        log2("......call to Bootstrap.virtualMachineManager()");

        vmm = Bootstrap.virtualMachineManager();
        if (vmm == null) {
            log3("ERROR: null returned");
            testExitCode = FAILED;
        } else {

            log2("......call to vmm.connectedVirtualMachines()");
            connectedVM = vmm.connectedVirtualMachines();

            size = connectedVM.size();
            if (size != 1) {
                log3("ERROR: connectedVM.size() != 1 : " + size);
                testExitCode = FAILED;
            } else {
                log2("       connectedVM.size() == 1");
                log2("......getting: VirtualMachine vm1 = connectedVM.get(0);");
                vm1 = (VirtualMachine) connectedVM.get(0);

                log2("......comparing: vm.equals(vm1)");
                if (!vm.equals(vm1)) {
                    log3("ERROR: !vm.equals(vm1)");
                    testExitCode = FAILED;
                }
            }
        }

        eventQueue = vm.eventQueue();
        eventSet = null;

        log2("......vm1.exit(" + (PASSED + PASS_BASE) + ")");
        vm1.exit(PASSED + PASS_BASE);

        log2("waiting for the debuggee to finish ...");
        debuggee.waitFor();

        int status = debuggee.getStatus();
        if (status != PASSED + PASS_BASE) {
            log3("debuggee returned UNEXPECTED exit status: " + status + " != PASS_BASE");
            testExitCode = FAILED;
        } else {
            log2("debuggee returned expected exit status: " + status + " == PASS_BASE");
        }

        if (vmDisconnect() == 0) {
            log2("......connectedVM = vmm.connectedVirtualMachines();");
            connectedVM = vmm.connectedVirtualMachines();

            size = connectedVM.size();
            if (size != 0) {
                log3("ERROR: connectedVM.size() != 0 : " + size);
                testExitCode = FAILED;

                log2("......since a returned list is not empty");
                log2("......try to get first element and compare to the mirror of the vm tested");
                try {
                    log2("        getting: vm1 = (VirtualMachine) connectedVM.get(0);");
                    vm1 = (VirtualMachine) connectedVM.get(0);

                    log2("         vm.name()== " + vm1.name());
                    if (vm.equals(vm1))
                        log2("vm.equals(vm1)");
                    else
                        log2("!vm.equals(vm1)");
                } catch (Exception e) {
                    log2("......failure because of Exception:  " + e);
                }

            } else
                log2("       connectedVM.size() == 0");
        } else {
            log3("ERROR: threadDeath != 0");
            testExitCode = FAILED;
        }

        log1("      TESTING ENDS");

        if (testExitCode != PASSED) {
            logHandler.complain("TEST FAILED");
        }
        return testExitCode;
    }

    private int vmDisconnect() {

        int returnCode = returnCode0;

        log2("       waiting for VMDisconnectEvent");

        labelBP: for (;;) {

            log2("       new:  eventSet = eventQueue.remove();");
            try {
                eventSet = eventQueue.remove(waitTime * 60000);
                if (eventSet == null) {
                    log3("ERROR:  timeout for waiting for a ThreadDeathEvent");
                    returnCode = returnCode3;
                    break labelBP;
                }
            } catch (Exception e) {
                log3("ERROR: Exception for  eventSet = eventQueue.remove(); : " + e);
                returnCode = 1;
                break labelBP;
            }

            if (eventSet != null) {

                log2("     :  eventSet != null;  size == " + eventSet.size());

                EventIterator eIter = eventSet.eventIterator();
                Event ev = null;

                for (; eIter.hasNext();) {

                    if (returnCode != returnCode0)
                        break;

                    ev = eIter.nextEvent();

                    ll: for (int ifor = 0;; ifor++) {

                        try {
                            switch (ifor) {

                            case 0:
                                AccessWatchpointEvent awe = (AccessWatchpointEvent) ev;
                                log2("      AccessWatchpointEvent removed");
                                break ll;
                            case 1:
                                BreakpointEvent be = (BreakpointEvent) ev;
                                log2("      BreakpointEvent removed");
                                break ll;
                            case 2:
                                ClassPrepareEvent cpe = (ClassPrepareEvent) ev;
                                log2("      ClassPreparEvent removed");
                                break ll;
                            case 3:
                                ClassUnloadEvent cue = (ClassUnloadEvent) ev;
                                log2("      ClassUnloadEvent removed");
                                break ll;
                            case 4:
                                ExceptionEvent ee = (ExceptionEvent) ev;
                                log2("      ExceptionEvent removed");
                                break ll;
                            case 5:
                                MethodEntryEvent mene = (MethodEntryEvent) ev;
                                log2("      MethodEntryEvent removed");
                                break ll;
                            case 6:
                                MethodExitEvent mexe = (MethodExitEvent) ev;
                                log2("      MethodExiEvent removed");
                                break ll;
                            case 7:
                                ModificationWatchpointEvent mwe = (ModificationWatchpointEvent) ev;
                                log2("      ModificationWatchpointEvent removed");
                                break ll;
                            case 8:
                                StepEvent se = (StepEvent) ev;
                                log2("      StepEvent removed");
                                break ll;
                            case 9:
                                ThreadDeathEvent tde = (ThreadDeathEvent) ev;
                                log2("      ThreadDeathEvent removed");
                                break ll;
                            case 10:
                                ThreadStartEvent tse = (ThreadStartEvent) ev;
                                log2("      ThreadStartEvent removed");
                                break ll;
                            case 11:
                                VMDeathEvent vmde = (VMDeathEvent) ev;
                                log2("      VMDeathEvent removed");
                                break ll;
                            case 12:
                                VMStartEvent vmse = (VMStartEvent) ev;
                                log2("      VMStartEvent removed");
                                break ll;
                            case 13:
                                WatchpointEvent we = (WatchpointEvent) ev;
                                log2("      WatchpointEvent removed");
                                break ll;

                            case 14:
                                VMDisconnectEvent wmde = (VMDisconnectEvent) ev;
                                log2("      VMDisconnectEvent removed");
                                break labelBP;

                            default:
                                log3("ERROR:  default case for casting event");
                                returnCode = returnCode3;
                                break ll;
                            } // switch
                        } catch (ClassCastException e) {
                        } // try
                    } // ll: for (int ifor =0;  ; ifor++)
                } // for (; ev.hasNext(); )
            }
        }
        if (returnCode == returnCode0)
            log2("     :  eventSet == null:  EventQueue is empty");

        return returnCode;
    }

}
