/*
 * Copyright (c) 2015, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * Native test for ToStringInInterfaceTest.
 */

#include "jni.h"

#define checkException(env) if ((*env)->ExceptionCheck(env)) { return; }

jstring callStringMethod(JNIEnv* env, jobject jobj, jmethodID id, ...)
{
    jstring value;

    va_list ap;
    va_start(ap, id);
    value = (jstring)(*env)->CallObjectMethodV(env, jobj, id, ap);
    va_end(ap);
    return value;
}

JNIEXPORT void JNICALL Java_ToStringTest_nTest(JNIEnv* env, jclass jclazz)
{
    jclass classOfInterfaceWithToString;
    jclass classOfImplementationOfWithToString;
    jmethodID constructorOfImplementationOfWithToString;
    jobject instanceOfImplementationOfWithToString;
    jmethodID toStringOfInterfaceWithToString;
    jmethodID toStringOfImplementationOfWithToString;
    jstring jstr;
    const char *chars;

    classOfInterfaceWithToString = (*env)->FindClass(env, "InterfaceWithToString");
    checkException(env);
    classOfImplementationOfWithToString = (*env)->FindClass(env, "ImplementationOfWithToString");
    checkException(env);

    constructorOfImplementationOfWithToString = (*env)->GetMethodID(env, classOfImplementationOfWithToString, "<init>", "()V");
    checkException(env);

    instanceOfImplementationOfWithToString = (*env)->NewObject(env, classOfImplementationOfWithToString, constructorOfImplementationOfWithToString);
    checkException(env);

    toStringOfInterfaceWithToString = (*env)->GetMethodID(env, classOfInterfaceWithToString, "toString", "()Ljava/lang/String;");
    checkException(env);

    toStringOfImplementationOfWithToString = (*env)->GetMethodID(env, classOfImplementationOfWithToString, "toString", "()Ljava/lang/String;");
    checkException(env);

    jstr = callStringMethod(env, instanceOfImplementationOfWithToString, toStringOfImplementationOfWithToString);
    checkException(env);

    chars = (*env)->GetStringUTFChars(env, jstr, NULL);
    (*env)->ReleaseStringUTFChars(env, jstr, chars);

    jstr = callStringMethod(env, instanceOfImplementationOfWithToString, toStringOfInterfaceWithToString);
    checkException(env);

    chars = (*env)->GetStringUTFChars(env, jstr, NULL);
    (*env)->ReleaseStringUTFChars(env, jstr, chars);
}
