/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
/*
 * EventObject.java
 *
 * Created on June 3, 2005, 1:27 PM
 */

package nsk.share.jvmti.attach.loader;

/**
 * This is a dummy object which is created by the ClEventHelper class
 * in its static method. It holds a single String,
 * Creating this object will make one more call back event to the
 * (VM) native agent.
 */

public class EventObject {
    private String name = null;
    private byte[] hugeBuffer;

    public EventObject(String name) {
        this.name = name;
    }

    public void setBreakPointHere() {
        System.out.println(" Here Default Break point can be set");
    }

    public void createHeap(int size) {
        hugeBuffer = new byte[size];
    }
    public void unAllocateMem() {
        hugeBuffer = null;
        System.gc();
    }


}
