/*
 * Copyright (c) 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package compiler.c2;

/*
 * @test
 * @bug 8314191
 * @summary Loop increment should not be transformed into unsigned comparison
 *
 * @run main/othervm -Xcomp -XX:-TieredCompilation
 *                   -XX:CompileCommand=compileonly,*MinValueStrideCountedLoop::test*
 *                   compiler.c2.MinValueStrideCountedLoop
 */

/*
 * @test
 * @bug 8316719
 * @summary Loop increment should not be transformed into unsigned comparison
 * @requires vm.compiler2.enabled
 * @run main/othervm -Xcomp -XX:-TieredCompilation -XX:-UseLoopPredicate
 *                   -XX:CompileCommand=compileonly,*MinValueStrideCountedLoop::test*
 *                   compiler.c2.MinValueStrideCountedLoop
 */
public class MinValueStrideCountedLoop {
    static int limit = 0;
    static int res = 0;
    static int[] array = new int[1];
    static boolean b;

    static void test1() {
        for (int i = 0; i >= limit + -2147483647; i += -2147483648) {
            res += 42;
        }
    }

    static int test2(int init, int limit) {
        int res = 0;
        int i = init;
        do {
            if (b) { }
            res += array[i];
            i += -2147483648;
        } while (i >= limit + -2147483647);
        return res;
    }

    public static void main(String[] args) {
        test1();
        test2(0, 0);
    }
}
