/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package nsk.share.jvmti.RetransformClasses.LinearHierarchy;

public class Class1 extends Class2 {
    final double var_1 = 1.0709268827590083E308;

    public Class1()
    {
        try {
            this.test();
        } catch (Throwable e) {}
    }

    private void test()
    {
        "rywxpfax".concat("gtnf");
        {
            "mv".offsetByCodePoints((char)(byte)1594457115 << 2203310960623951872L, (short)'S' << 1832042558);
        }
        "rvekd".indexOf("bldp".offsetByCodePoints(876767138, 1063197859 | ~ ((byte)701606985)), (!true ? (byte)1.4903633E38F : (byte)var_1) + (byte)((+ + -2.571243E38F + (short)var_1) / 'D'));
        long var_2 = 0L;
        new String();
        for (short var_3 = 4569; (true ? !true : true) && var_2 < 22; var_3 -= (byte)(("e" + "lhod".concat("jlimetf")).endsWith("awmx") | 1486245383 < -var_1 - 1.7333834784857044E308 ? (byte)5.744466E36F : + - ~ ~4452397653252341760L) / (false | false | true ? (byte)'R' : (byte)1303106408135818240L))
        {
            boolean var_4;
            var_2++;
            var_4 = (var_4 = !false & true);
            var_4 = (var_4 = "".endsWith("".toUpperCase()));
        }
        int var_5 = 0;
        "t".lastIndexOf(false | ! !false & !false ? 'c' & (byte)2.1659184E38F : (short)'s', 136261);
        while (false && (var_5 < 167 && ((var_1 > (byte)var_5 ? true : 'C' < "".indexOf("")) || (byte)1.2533985E37F != (byte)var_2)))
        {
            new String();
            var_5++;
            (((byte)2.0544619713409872E307 % (byte)-2.6472738E37F == - ((byte)'S') & (!true ^ !true) ? !true : false) ? "tuf" : "ydiys").concat("juutd");
            ((true ? true : "d".startsWith("mbqvluc", var_5)) ? "oijlsjmu" : "wtdtjty").endsWith("esc");
        }
        int var_6 = 0;
        (! !true | 'f' < (short)var_6 || true ? "ljveqrshi" : "").indexOf("bume", 1073767872);
        do
        {
            ((false ? (short)'o' : (byte)(short)var_6) < 8.641181359683153E307 ? "edesvvqx" : "addvn").toString().startsWith("weqp", var_6);
            var_6++;
            "cwxpkyma".indexOf("h");
        } while (var_6 < 198);
        (false && true ? "gciq".trim().replaceFirst("qp", "gmiibgb") : "ovxpcc").equalsIgnoreCase("nrqgx");
        (((short)'I' & 'F') == (true ? (byte)(char)5.095923109596564E307 : (short)'\\') ? "pvctfv" : "ahga").hashCode();
    }
}
