/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package nsk.share.gc.gp;

/**
 * Garbage producer that uses another garbage producer
 * to implement it's functionality.
 */
public abstract class DerivedProducer<T, P> implements GarbageProducer<T> {
        private GarbageProducer<P> parent;

        public DerivedProducer(GarbageProducer<P> parent) {
                setParent(parent);
        }

        protected P createParent(long memory) {
                return parent.create(memory);
        }

        protected void validateParent(P obj) {
                parent.validate(obj);
        }

        public final GarbageProducer<P> getParent() {
                return parent;
        }

        public final void setParent(GarbageProducer<P> parent) {
                this.parent = parent;
        }
}
