/*
 * Copyright (c) 2022, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test
 * bug 8280600
 * @summary C2: assert(!had_error) failed: bad dominance
 * @run main/othervm -Xcomp -XX:CompileOnly=TestCastIIMakesMainLoopPhiDead2::* TestCastIIMakesMainLoopPhiDead2
 */

public class TestCastIIMakesMainLoopPhiDead2 {
    static int zero = 0;

    public static void main(String[] args) {
        for (int i = 0; i < 100000; i++) {
            test();
        }
    }

    static void test() {
        int h[] = new int[zero];
        for (int m = 0; m < 5; m++) {
            try {
                for (int f = -400; f < 1; f++) {
                    h[f] = 1; // Out of bounds store.
                }
            } catch (ArrayIndexOutOfBoundsException i) {
                // Expected
            }
        }
    }
}
