/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package nsk.share.jvmti.RetransformClasses.LinearHierarchy;

public class Class8 extends Class9 {
    int var_1 = 1731200429;
    long var_2 = 8420857753000168448L;
    final double var_3 = 1.7922227805230007E307;
    byte var_4;


    public Class8()
    {
        try {
            this.test();
        } catch (Throwable e) {}
    }

    private void test()
    {
        float var_5 = 0F;
        {
            int var_6;
        }
        var_1 /= (short)(var_4 = (byte)var_2) >>> (short)5.3159896E37F;
        var_2--;
        var_2 %= var_3;
        var_4 = (("cayhyaww" + "fmcfljl".replaceFirst("deekbwwlo", "ggkc")).endsWith("v") ? true : !false | !true) ? (var_4 = (var_4 = (byte)var_2)) : (var_4 = (var_4 = (var_4 = (byte)var_1)));
        {
            var_4 = (var_4 = false & true ? (byte)'c' : (byte)var_5);
        }
        "tmykbv".substring((true & false ? (byte)9.086589E37F : (byte)(short)'y') + 'B');
        var_4 = ! (true | true & false) ^ var_1 < (short)1.1118761317216113E308 ^ !false ? (var_4 = (var_4 = (var_4 = (byte)var_1))) : (byte)(var_2 /= var_5);
        while (var_5 < 129)
        {
            var_1 *= (short)1468549302698859520L <= (var_1 &= var_1) ? 'R' : 'A';
            {
                var_4 = (var_4 = (var_4 = (var_4 = (byte)((short)var_2 - 1.7169119E38F))));
                var_1--;
            }
            final double var_7 = var_3;
            int var_8;
            var_2 <<= (short)var_3;
            float var_9;
            byte var_10 = var_4 = (var_4 = (var_4 = (byte)var_5));
            var_9 = 'J' >>> var_10;
            final char var_11 = 'I';
            var_1 &= (short)(var_5 - (false | !false | !false ? var_10 : (short)(var_10 /= var_10)));
            var_10 ^= var_11;
            var_9 = 958907235 == var_11 % ~var_10 ? 'y' : var_11;
            var_5++;
            char var_12 = 'S';
            {
                var_10 %= (var_12 >>>= - ((var_2 -= 1349535968) % var_10));
            }
            var_12--;
            var_9 = (var_2 |= (var_12 >>= var_12));
            var_10 >>= (8.412603968810148E307 < ++var_1 | !false ? !true : false) ? (short)var_12++ : var_10;
            var_9 = var_1;
            var_12--;
            var_1 *= var_2;
            var_9 = ((!false && ! !true) | true ? !true : !false) ? (short)var_1 : (var_10 /= var_12);
            {
                var_2 &= var_10;
                {
                    var_9 = 1743064098;
                }
                var_10 ^= 1958573961;
                var_12 *= 6222637042293534720L;
            }
            var_2 /= 2845605111873083392L;
            var_1 |= var_10;
            float var_13;
            ++var_12;
            var_13 = (short)~ (var_10 |= (char)168602744);
            var_1++;
            {
                var_1--;
            }
            var_10 /= 9.60625830185491E307;
            final long var_14 = 5545095530950148096L;
        }
        var_2 -= false ^ 'D' > 6.197060025668609E307 & (true | !false) ? (short)769680980 : (short)var_3;
        var_4 = '`' <= ((var_1 %= (byte)'g') == 9.257869495808238E307 ? (byte)var_1 : (byte)1.1342762499693523E308) ? (var_4 = (var_4 = (byte)var_1)) : (byte)var_3;
        if (true)
        {
            var_1 >>>= (var_4 = (byte)'X');
        }
        else
        {
            if (false)
            {
                var_4 = (var_4 = (byte)5871571285290165248L);
            }
            else
            {
                {
                    var_4 = ! (1.6268232E38F <= (short)((char)2035109601 & (short)var_1) | true) ? (var_4 = (byte)var_3) : (byte)(-var_2 << "qqatqt".charAt(1351097548));
                    var_2 >>>= "sy".charAt(~ (var_4 = (byte)143394786396589056L));
                }
                var_4 = ((! (("iqosmy".equalsIgnoreCase("sdobty") | var_2 > (var_2 = (byte)'Q')) & ! !true) ? !true : !true) ? ! !false : (false | !true) & false) ^ !false & !false ? (var_4 = (byte)123259801) : (var_4 = (var_4 = (byte)'Y'));
                var_1 >>>= 1538858488 != 'A' + (short)var_3 ? (byte)'Z' : (byte)var_5;
                var_1 <<= 5120038810497601536L;
                float var_15;
                --var_1;
                ('d' != ((var_15 = 3.1916732E38F) < 2.948598E38F && true ? (short)~ ~var_2 : (byte)var_5) && (false && !"tvjpk".endsWith("umbflc") ^ true ^ false) ? "v" : "rusqhes").length();
                var_4 = (var_4 = true || -1.1068047224709673E308 != (byte)1.0449807E38F ? (var_4 = (byte)++var_2) : (var_4 = (var_4 = (byte)5.907563648481024E307)));
                var_15 = var_1;
                String var_16;
            }
            long var_17 = 0L;
            var_4 = (var_4 = (byte)var_3);
            while ((+var_5 >= (var_4 = (byte)1.7473458E38F) ? true : false) | true && (var_17 < 39 && false ^ (!false | false) ^ (false || true & !false)))
            {
                int var_18;
                var_17++;
                byte var_19;
                float var_20 = var_5;
            }
            (("ssir" + (! ('l' < (byte)((short)var_5 - "wdiglndlt".charAt(247773955))) ? "weqhpc" : "lnmpnuhue")).startsWith(("".endsWith("") ? "tonxlb" : "jrwvh") + "mamcse") ? (false | !false ? "gd" : new String()) : "lv" + "owee").trim();
            var_4 = (byte)1405838286;
            var_2 &= (var_4 = (byte)'E');
            long var_21 = 0L;
            var_2 <<= (byte)(short)'G' % (byte)(short)var_3;
            do
            {
                var_2 = (var_4 = (byte)'G');
                var_21++;
                var_1 ^= var_21;
            } while (var_21 < 17 && ('F' == (var_1 <= ++var_1 ? - +1.194147567807097E308 : var_3) ? "vcq".startsWith("oebypfcmo", var_1) : true));
            var_4 = (var_4 = (var_4 = (var_4 = (byte)'K')));
            var_1 >>>= (short)(byte)(short)(var_2 & var_2);
            --var_1;
            var_1 *= ("" + "").charAt((short)1183271275 << (char)var_2);
            var_4 = (byte)(short)'I';
            var_1 -= 1495333912;
            var_2 |= 640605392;
            var_1 >>= ("".startsWith("yycybrcq") ? true : false && var_3 > (byte)var_17) ? (var_4 = (var_4 = (var_4 = (byte)-2.7770478E38F))) : (byte)(short)var_5;
            "x".indexOf(var_1);
            var_4 = (var_4 = "".endsWith("") & false ? (byte)~var_2 : (var_4 = (byte)(char)-7561815264392892416L));
            var_4 = true ? (byte)var_1 : (var_4 = (var_4 = (byte)var_2));
            long var_22 = 0L;
            var_4 = (var_4 = false & !true ? (byte)'e' : (byte)'W');
            while (!false && var_22 < 17)
            {
                final short var_23 = 29046;
                var_22++;
                var_4 = (var_4 = (true ? !true : !true) ? (byte)8575391698399610880L : (byte)var_22);
                var_1 >>>= var_23;
            }
            var_1 &= "xwj".charAt('D' << 'p');
            var_2 |= (! !true | false) & false ? 'c' : 'T';
            boolean var_24;
            var_4 = (var_4 = (var_4 = (var_4 = (byte)5.296192969367163E307)));
        }
        {
            Object var_25;
        }
        switch (((false ^ (byte)var_5 != (var_2 ^= var_1)) & (false ^ (short)5717933608817793024L == - + ((float)~var_2)) || false ? ! (false & !false) : !false) ? (short)("hh".substring(594610734).endsWith("qwjp") ? 'o' : 'X') : (short)(byte)var_5)
        {
            case 78:

            case 61:

            case 121:

            case 69:

        }
        var_4 = (var_4 = (var_4 = (byte)'U'));
        var_2 = 4478167756956753920L * var_1;
        final short var_26 = ((true ^ true ? false : !false) ? !true & !false | 2.2989038E38F > -var_3 : -var_3 != 'J') ? (short)'R' : (short)var_5;
    }
}
