/*
 * Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package nsk.share;

import java.util.Properties;

/**
 * This program prints system properties.
 */
public class PrintProperties {
        public static void main(String[] args) {
                Properties pr = System.getProperties();
                switch (args.length) {
                case 0:
                        pr.list(System.out);
                        System.exit(0);
                case 1:
                        String value = pr.getProperty(args[0]);
                        if (value == null) {
                                System.err.println("Not found");
                                System.exit(1);
                        } else {
                                System.out.println(value);
                                System.exit(0);
                        }
                default:
                        System.out.println("Usage:");
                        System.out.println("    PrintProperties");
                        System.out.println("    PrintProperties <property name>");
                        System.exit(255);
                }
        }
}
