/*
 * Copyright (c) 2001, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package nsk.jdi.ExceptionRequest.addInstanceFilter;

import nsk.share.*;
import nsk.share.jpda.*;
import nsk.share.jdi.*;

import com.sun.jdi.*;
import com.sun.jdi.event.*;
import com.sun.jdi.request.*;

import java.util.*;
import java.io.*;

/**
 * The test for the implementation of an object of the type
 * ExceptionRequest.
 *
 * The test checks that results of the method
 * <code>com.sun.jdi.ExceptionRequest.addInstanceFilter()</code>
 * complies with its spec.
 *
 * The test checks up on the following assertion:
 *    Restricts the events generated by this request to those
 *    in which the currently executing instance is
 *    the object specified.
 *
 * The test has works as follows.
 * - The debugger resumes the debuggee and waits for the BreakpointEvent.
 * - The debuggee creates a special array with three Object instances,
 *   and two threads, thread1 and thread2, and invokes
 *   the methodForCommunication to be suspended and
 *   to inform the debugger with the event.
 * - Upon getting the BreakpointEvent, the debugger
 *   - sets up a BreakpointRequest
 *     within the method in the class TestClass
 *     whose array element instances #0 and #1
 *     will be calling by the thread1 and the thread2 accordinly,
 *   - invokes addInstanceFilter() on array element #0 for the thread1
 *     and #2 for the thread2,
 *     thus restricting the breakpoint event only to thread1,
 *   - resumes debuggee's main thread, and
 *   - waits for the event.
 * - Debuggee's main thread starts both threads.
 * - Upon getting the event, the debugger performs the checks required.
 */

public class instancefilter001 extends TestDebuggerType1 {

    public static void main (String argv[]) {
        int result = run(argv,System.out);
        if (result != 0) {
            throw new RuntimeException("TEST FAILED with result " + result);
        }
    }

    public static int run (String argv[], PrintStream out) {
        debuggeeName = "nsk.jdi.ExceptionRequest.addInstanceFilter.instancefilter001a";
        return new instancefilter001().runThis(argv, out);
    }

    private String testedClassName =
      "nsk.jdi.ExceptionRequest.addInstanceFilter.instancefilter001aTestClass";

    protected void testRun() {
        if ( !vm.canUseInstanceFilters() ) {
            display("......vm.canUseInstanceFilters == false :: test cancelled");
            vm.exit(Consts.JCK_STATUS_BASE);
            return;
        }

        EventRequest eventRequest1 = null;
        EventRequest eventRequest2 = null;

        ThreadReference thread1 = null;
        ThreadReference thread2 = null;

        String thread1Name = "thread1";
        String thread2Name = "thread2";

        String property1 = "ExceptionRequest1";
        String property2 = "ExceptionRequest2";

        ReferenceType testClassReference = null;

        String         arrayName = "objTC";
        ObjectReference instance = null;

        for (int i = 0; ; i++) {

            if (!shouldRunAfterBreakpoint()) {
                vm.resume();
                break;
            }

            display(":::::: case: # " + i);

            switch (i) {

                case 0:
                testClassReference =
                     (ReferenceType) vm.classesByName(testedClassName).get(0);

                thread1 = (ThreadReference) debuggeeClass.getValue(
                                            debuggeeClass.fieldByName(thread1Name));
                eventRequest1 = setting2ExceptionRequest (thread1,
                                       testClassReference,
                                       EventRequest.SUSPEND_ALL, property1);
                instance = (ObjectReference)
                           ((ArrayReference) debuggeeClass.getValue(
                           debuggeeClass.fieldByName(arrayName)) ).getValue(0);
                ((ExceptionRequest) eventRequest1).addInstanceFilter(instance);

                thread2 = (ThreadReference) debuggeeClass.getValue(
                                            debuggeeClass.fieldByName(thread2Name));
                eventRequest2 = setting2ExceptionRequest (thread2,
                                       testClassReference,
                                       EventRequest.SUSPEND_ALL, property2);
                instance = (ObjectReference)
                           ((ArrayReference) debuggeeClass.getValue(
                           debuggeeClass.fieldByName(arrayName)) ).getValue(2);
                ((ExceptionRequest) eventRequest2).addInstanceFilter(instance);

                display("......waiting for ExceptionEvent in expected thread");

                Event newEvent = eventHandler.waitForRequestedEvent(new EventRequest[]{eventRequest1, eventRequest2}, waitTime, true);

                if ( !(newEvent instanceof ExceptionEvent)) {
                    setFailedStatus("ERROR: new event is not ExceptionEvent");
                } else {

                    String property = (String) newEvent.request().getProperty("number");
                    display("       got new ExceptionEvent with property 'number' == " + property);

                    if ( !property.equals(property1) ) {
                        setFailedStatus("ERROR: property is not : " + property1);
                    }
                }

                vm.resume();
                break;

                default:
                throw new Failure("** default case 2 **");
            }
        }
        return;
    }

    private ExceptionRequest setting2ExceptionRequest ( ThreadReference thread,
                                                       ReferenceType   testedClass,
                                                       int             suspendPolicy,
                                                       String          property       ) {
        try {
            display("......setting up ExceptionRequest:");
            display("       thread: " + thread + "; class: " + testedClass + "; property: " + property);

            ExceptionRequest
            excr = eventRManager.createExceptionRequest(null, true, true);
            excr.putProperty("number", property);
            if (thread != null)
                excr.addThreadFilter(thread);
            excr.addClassFilter(testedClass);
            excr.setSuspendPolicy(suspendPolicy);

            display("      ExceptionRequest has been set up");
            return excr;
        } catch ( Exception e ) {
            throw new Failure("** FAILURE to set up ExceptionRequest **");
        }
    }
}
