/*
 * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 *
 */

/**
 * @test
 * @summary Test 'iload_w' bytecode in shared class
 * @requires vm.cds
 * @library /test/lib
 * @compile test-classes/Iloadw.jasm
 * @compile test-classes/IloadwMain.java
 * @run driver WideIloadTest
 */

import jdk.test.lib.process.OutputAnalyzer;

public class WideIloadTest {
    public static void main(String args[]) throws Exception {
        JarBuilder.build("iload_w", "Iloadw", "IloadwMain");
        String appJar = TestCommon.getTestJar("iload_w.jar");
        OutputAnalyzer dumpOutput = TestCommon.dump(appJar, TestCommon.list(
                                        "Iloadw", "IloadwMain"));
        TestCommon.checkDump(dumpOutput);
        OutputAnalyzer execOutput = TestCommon.exec(appJar, "IloadwMain");
        TestCommon.checkExec(execOutput, "Passed");
    }
}
