/*
 * Copyright (c) 2020, 2024, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

package gc.z;

/*
 * @test TestAllocateHeapAt
 * @requires vm.gc.Z & os.family == "linux"
 * @requires !vm.opt.final.UseLargePages
 * @requires !vm.opt.final.UseTransparentHugePages
 * @summary Test ZGC with -XX:AllocateHeapAt
 * @library /test/lib
 * @run main/othervm gc.z.TestAllocateHeapAt . true
 * @run main/othervm gc.z.TestAllocateHeapAt non-existing-directory false
 */

import jdk.test.lib.os.linux.HugePageConfiguration;
import jdk.test.lib.os.linux.HugePageConfiguration.ShmemTHPMode;
import jdk.test.lib.process.ProcessTools;
import jtreg.SkippedException;

public class TestAllocateHeapAt {
    public static void main(String[] args) throws Exception {
        final String directory = args[0];
        final boolean exists = Boolean.parseBoolean(args[1]);
        final String heapBackingFile = "Heap Backing File: " + directory;
        final String failedToCreateFile = "Failed to create file " + directory;

        final HugePageConfiguration hugePageConfiguration = HugePageConfiguration.readFromOS();
        final ShmemTHPMode mode = hugePageConfiguration.getShmemThpMode();

        if (mode != ShmemTHPMode.never && mode != ShmemTHPMode.advise) {
            throw new SkippedException("The UseTransparentHugePages option may not be respected with Shmem THP Mode: " + mode.name());
        }

        ProcessTools.executeTestJava(
            "-XX:+UseZGC",
            "-Xlog:gc*",
            "-Xms32M",
            "-Xmx32M",
            "-XX:AllocateHeapAt=" + directory,
            "-version")
                .shouldContain(exists ? heapBackingFile : failedToCreateFile)
                .shouldNotContain(exists ? failedToCreateFile : heapBackingFile)
                .shouldHaveExitValue(exists ? 0 : 1);
    }
}
