/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/*
 * @test TestClassResolutionFail
 * @bug 8023697
 * @summary This tests that failed class resolution doesn't report different class name in detail message for the first and subsequent times
 */

import java.io.File;

public class TestClassResolutionFail {
    static String message;
    public static void test1() throws RuntimeException {
        try {
            Property p = new Property();
        } catch (LinkageError e) {
            message = e.getMessage();
        }
        try {
            Property p = new Property();
        } catch (LinkageError e) {
            System.out.println(e.getMessage());
            if (!e.getMessage().equals(message)) {
                throw new RuntimeException("Wrong message: " + message + " != " + e.getMessage());
            }
        }
    }
    public static void main(java.lang.String[] unused) throws Exception {
        // Remove PropertySuper class
        String testClasses = System.getProperty("test.classes", ".");
        File f = new File(testClasses + File.separator + "PropertySuper.class");
        f.delete();
        test1();
    }
}

