/*
 * Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
package nsk.share.jvmti.RetransformClasses.LinearHierarchy;

public class Class7 extends Class8 {
    double var_1 = 1.4699591272215534E308;
    final String var_2 = "pxkiwbqo";
    byte var_3 = 2.185587E38F > 3654315426407157760L >> (false ? -1565206782971839488L : ~3474492181971301376L) ? (byte)1.5672585601100038E308 : (byte)522173992;
    char var_4 = 'T';
    char var_5;
    String var_6 = "gn";
    byte var_7;
    double var_8;
    final boolean var_9 = false;
    int var_10;


    public Class7()
    {
        try {
            this.test();
        } catch (Throwable e) {}
    }

    private void test()
    {
        double var_11 = 0;
        var_6 = false & true ^ var_9 ? "jc" : "pm";
        String var_12 = "fcrqfyn";
        var_4 ^= false ? (var_3 *= var_3) : var_3;
        do
        {
            var_3 += 4.59478803152362E307;
            var_12 = var_6;
            var_1 *= true ^ false || var_9 ? (short)var_3++ : (short)~ (- ~ + ~285357738466553856L % (var_3 ^= 'm'));
            var_10 = var_4 != (var_10 = (short)6.474266043423393E307) ? var_3 : (short)(var_10 = 988736393);
            final long var_13 = 954458749486001152L;
            var_11++;
            var_10 = 942430176;
        } while (var_11 < 286);
        float var_14 = 0F;
        var_10 = var_9 ? (short)var_1 : (var_3 ^= 1019475088);
        var_10 = var_3;
        var_1 *= (short)var_14;
        var_1 %= (short)978731980;
        for (String var_15 = "nwyyxsh"; var_14 < 46 && !var_9 | var_9; var_6 = (false | "eieg".endsWith(var_2)) ^ var_9 ? var_2 : "ekycjyrl")
        {
            var_3 ^= 7946659604163515392L != (var_4 ^= ~ (+9198633046707913728L >> "qtvdekoik".lastIndexOf(var_2))) ? (var_4 += 1673759362) : var_4;
            {
                var_8 = (short)((var_9 ? var_9 | false && var_9 : true) ? var_14 : var_4);
            }
            var_10 = (int)(var_3 >>>= (var_4 %= 1526142557));
            var_4 &= !var_9 | var_6.charAt(1717811369) < (var_9 ^ !var_9 ? var_3 : (short)2077195010) || var_9 ? var_3 : (var_3 = var_3);
            var_14++;
            var_1 %= ! (true ^ ((!true | (false | true)) & var_9 ? !"nuvgubfy".startsWith(var_15) : ! !false)) ? ++var_3 : (short)(float)var_3;
            var_4 >>>= false ^ !true ^ var_9 & (6199879605438218240L ^ var_3) == 1426867424 ? (char)((var_3 ^= var_4) >> - (var_3 /= var_14)) : (var_5 = 'j') % (short)(~1646044248523242496L >>> 'd') + var_4;
        }
        if (true)
        {
            var_1 += (short)(true & (!true && var_9) | (false ^ !var_9) & (var_9 ^ var_9) ? var_3 : var_3);
            var_8 = 1.2587289E37F + var_3--;
        }
        else
        {
            var_3 <<= true ? + +5608251798734573568L : (short)var_3;
        }
        char var_16 = '^';
        final int var_17 = 2062936021;
        var_8 = 'f';
    }
}
